//================================================================
//=	  Obsluga RTC PCF8583
//=	  CPU = ATMega8515
//=   CLK = 8MHz
//================================================================

#include <avr/io.h>
#include <avr/interrupt.h>
#include "vector_disp.h"
#include "i2c.h"
#include "PCF8583.h"


extern volatile unsigned char czas[7]; 			// g,g,m,m,s,"godziny" i "minuty" (liczba stopni dla zegara analogowego)


//================================================================
//===== Inicjalizacja RTC ========================================
//================================================================
void PCF8583_init(void)
{
	PORTD &=~ ((1 << 0) | (1 << 1));			// stan niski na SDA i SCL

	DDRD &=~ (1 << 2);							// PD.2 jako wejscie 1Hz

	MCUCR |= (1 << 1);							// INT0 wyzwalane zboczem opadajacym
	MCUCR &=~(1 << 0);

	GICR |= (1 << 6);							// wlaczenie przerwania INT0
}
//================================================================



//================================================================
//===== Obsluga RTC ==============================================
//================================================================
void wyslij_godzine(void)
{
	unsigned char bufor[5];

	bufor[0] = 0;								// control
	bufor[1] = 0;								// setne sekundy
	bufor[2] = 0;								// sekundy
	bufor[3] = (czas[2] << 4) | czas[3];		// minuty(10) i minuty(1)
	bufor[4] = (czas[0] << 4) | czas[1] | 128;	// tryb 12h, godziny(10) i godziny(1)
	
	I2C_Wyslij(ADRES,0,5,bufor);				// wyslij 5 bajtow od adresu 0x00
}


void pobierz_godzine(void)
{
	unsigned char bufor[2];

	I2C_Odbierz(ADRES,3,2,bufor);				// odbierz 2 bajty od adresu 0x03
	
	czas[0] = (bufor[1] & 0x30) >> 4;			// godziny(10)
	czas[1] = bufor[1] & 0x0F;					// godziny(1)
	czas[2] = bufor[0] >> 4;					// minuty(10)
	czas[3] = bufor[0] & 0x0F;					// minuty(1)
	
	czas[6] = (czas[2] * 10) + czas[3];			// minuty(10)x10 + minuty(1)
	czas[5] = (czas[1] + (czas[0] * 10));		// godziny (1 + 10)
	czas[5] = (czas[5] * 6) + czas[2];			// godziny * 6 + minuty(10)
}
//================================================================



//================================================================
//===== Przerwanie 1Hz ===========================================
//================================================================
ISR(INT0_vect)									// inkrementacja czasu co 1s
{
	czas[4]++;
	
	if(czas[4] > 59)							// minela minuta
	{
		czas[4] = 0;							// wyzeruj sekundy
		czas[3]++;								// zwieksz minuty(1)
		
		if(czas[3] > 9)							// minelo 10 minut
		{
			czas[3] = 0;						// wyzeruj minuty(1)
			czas[2]++;							// zwieksz minuty(10)
				
			if(czas[2] > 5)						// minela godzina
			{
				czas[2] = 0;					// wyzeruj minuty(10)
				czas[1]++;						// zwieksz godziny(1)
				
				if(czas[1] > 1)					// godzina(1) > 1
				{
					if(czas[0] == 1)			// godzina(10) == 1 => godzina 12
					{
						czas[0] = 0;			// wyzeruj godziny(10)
						czas[1] = 0;			// wyzeruj godziny(1)
					}
					else						// godziny(10) != 1
						if(czas[1] > 9)			// minelo 10 godzin	
						{
							czas[1] = 0;		// wyzeruj godziny(1)
							czas[0]++;			// zwieksz godziny(10)
						}
				}
			}	
		}
	}
	
	czas[6] = (czas[2] * 10) + czas[3];			// minuty(10)x10 + minuty(1)
	czas[5] = (czas[1] + (czas[0] * 10));		// godziny (1 + 10)
	czas[5] = (czas[5] * 6) + czas[2];			// godziny * 6 + minuty(10)
}
//================================================================





